# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ClosingRules',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('start', models.DateTimeField()),
                ('end', models.DateTimeField()),
                ('reason', models.TextField(null=True, blank=True)),
            ],
            options={
                'verbose_name': 'Closing Rule',
                'verbose_name_plural': 'Closing Rules',
            },
        ),
        migrations.CreateModel(
            name='Company',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('slug', models.SlugField(unique=True)),
                ('logo', models.FileField(null=True, upload_to=b'logo', blank=True)),
            ],
            options={
                'verbose_name': 'Company',
                'verbose_name_plural': 'Companies',
            },
        ),
        migrations.CreateModel(
            name='OpeningHours',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('weekday', models.IntegerField(choices=[(1, 'Monday'), (2, 'Tuesday'), (3, 'Wednesday'), (4, 'Thursday'), (5, 'Friday'), (6, 'Saturday'), (7, 'Sunday')])),
                ('from_hour', models.TimeField()),
                ('to_hour', models.TimeField()),
                ('company', models.ForeignKey(to='openinghours.Company')),
            ],
            options={
                'verbose_name': 'Opening Hour',
                'verbose_name_plural': 'Opening Hours',
            },
        ),
        migrations.AddField(
            model_name='closingrules',
            name='company',
            field=models.ForeignKey(to='openinghours.Company'),
        ),
    ]



import datetime

from django.conf import settings


try:
    from threadlocals.threadlocals import get_current_request
except ImportError:
    get_current_request = None


from openinghours.models import OpeningHours, ClosingRules, Company


def get_now():
    ''' '''
    # Allow access global request and read a timestamp from query...
    request = get_current_request()
    if request:
        openinghours_now = request.GET.get('openinghours-now')
        if openinghours_now:
            return datetime.datetime.strptime(openinghours_now, '%Y%m%d%H%M%S')
    return datetime.datetime.now()


def get_closing_rule_for_now(company_slug):
    '''
    Access the all closing rules for a company
    '''
    now = get_now()

    if company_slug:
        return ClosingRules.objects.filter(company__slug=company_slug, start__lte=now, end__gte=now)

    return Company.objects.first().closingrules_set.filter(start__lte=now, end__gte=now)
    
    
def has_closing_rule_for_now(company_slug):
    '''
    Has the company closing rules to evaluate?
    '''
    cr = get_closing_rule_for_now(company_slug)
    return cr.count()
    
    
def is_open(company_slug, now=None):
    '''
    Is the company currently open? Pass "now" to test with a specific timestamp.
    This method is used as stand alone and helper.
    '''
    if now is None:
        now = get_now()
  
    if has_closing_rule_for_now(company_slug):
        return False
        
    now_time = datetime.time(now.hour, now.minute, now.second)
    
    if company_slug:
        ohs = OpeningHours.objects.filter(company__slug=company_slug)
    else:
        ohs = Company.objects.first().openinghours_set.all()
    for oh in ohs:
        is_open = False
        # start and end is on the same day
        if oh.weekday == now.isoweekday() and oh.from_hour <= now_time and now_time <= oh.to_hour: 
           is_open = oh
        
        # start and end are not on the same day and we test on the start day
        if oh.weekday == now.isoweekday() and oh.from_hour <= now_time and ((oh.to_hour < oh.from_hour) and (now_time < datetime.time(23, 59, 59))):
            is_open = oh
            
        # start and end are not on the same day and we test on the end day
        if (oh.weekday == (now.isoweekday() - 1) % 7 and oh.from_hour >= now_time and oh.to_hour >= now_time and oh.to_hour < oh.from_hour):
            is_open = oh
            #print " 'Special' case after midnight", oh
        
        if is_open is not False:
            return oh
    return False
    
    
def next_time_open(company_slug):
    ''' 
    Returns the next possible opening hours object ( aka when is the company open for the next time?).
    '''
    if not is_open(company_slug):
        now = get_now()
        now_time = datetime.time(now.hour, now.minute, now.second)
        found_opening_hours = False
        for i in range(8):
            l_weekday = (now.isoweekday() + i) % 8
            ohs = OpeningHours.objects.filter(company__slug=company_slug, weekday=l_weekday).order_by('weekday','from_hour')
            
            if ohs.count():
                for oh in ohs:
                    future_now = now + datetime.timedelta(days=i)
                    # same day issue
                    tmp_now = datetime.datetime(future_now.year, future_now.month, future_now.day, oh.from_hour.hour, oh.from_hour.minute, oh.from_hour.second)
                    if tmp_now < now:
                        tmp_now = now # be sure to set the bound correctly...
                    if is_open(company_slug, now=tmp_now):
                        found_opening_hours = oh
                        break
                if found_opening_hours is not False:
                    return found_opening_hours, tmp_now
    return False, None



from django.contrib import admin
from openinghours.models import OpeningHours, ClosingRules, Company


admin.site.register(OpeningHours)
admin.site.register(ClosingRules)
admin.site.register(Company)



import datetime

from django import template
from django.utils.safestring import mark_safe
from django.utils.encoding import force_unicode 
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from openinghours.models import *
from openinghours.utils import *


register = template.Library()


@register.filter(expects_localtime=True)
def isoDayToWeekday(d):
    if int(d) == get_now().isoweekday():
        return _("today")
    for w in WEEKDAYS:
        if w[0] == int(d):
            return w[1]


@register.filter(expects_localtime=True)
def toWeekday(dateObjTpl):
    oh, date_obj = dateObjTpl
    now = get_now()
    if date_obj.isoweekday() == now.isoweekday() and (date_obj - now).days == 0:
        return _("today")
    for w in WEEKDAYS:
        if w[0] == int(date_obj.isoweekday()):
            return w[1]


@register.assignment_tag
def isCompanyCurrentlyOpen(company_slug=None, attr=None):
    obj = is_open(company_slug)
    if obj is False:
        return False
    if attr is not None:
        return getattr(obj, attr) 
    return obj

    
@register.filter(expects_localtime=True) 
def getCompanyNextOpeningHour(company_slug, attr=None):
    ''' 
    `attr` allowes to acces to a attribute of the OpeningHours model. 
    This is handy to access the start time for example...
    ''' 
    obj, ts = next_time_open(company_slug)
    if obj is False:
        return False 
    elif attr is not None:
        return getattr(obj, attr) 
    return obj, ts
    
    
@register.filter(expects_localtime=True) 
def has_closing_rule_for_now(company_slug, attr=None):
    obj = has_closing_rule_for_now(company_slug)
    if obj is False:
        return False
    if attr is not None:
        return getattr(obj, attr) 
    return obj


@register.filter(expects_localtime=True) 
def getCompanyClosingRuleForNow(company_slug, attr=None):
    ''' this only access the first! closing rule. because closed is closed. '''
    obj = get_closing_rule_for_now(company_slug)
    if obj is False:
        return False
    if attr is not None:
        return getattr(obj[0], attr) 
    return obj
               
    
@register.simple_tag
def companyOpeningHoursList(company_slug=None, concise=False):
    ''' Creates a rendered listing of hours. ''' 
    template_name = 'openinghours/companyOpeningHoursList.html'
    days = [] # [{'hours': '9:00am to 5:00pm', 'name': u'Monday'}, {'hours': '9:00am to...

    #If a `company_slug` is not provided, choose the first company.
    if company_slug: 
        ohrs = OpeningHours.objects.filter(company__slug=company_slug)
    else:
        try:
            ohrs = Company.objects.first().openinghours_set.all()
        except AttributeError:
            raise Exception("You must define some opening hours to use the opening hours tags.")

    ohrs.order_by('weekday', 'from_hour')

    for o in ohrs:
        days.append({
            'name': o.get_weekday_display(),
            'hours': '%s%s to %s%s' % (
                o.from_hour.strftime('%I:%M').lstrip('0'), 
                o.from_hour.strftime('%p').lower(),
                o.to_hour.strftime('%I:%M').lstrip('0'), 
                o.to_hour.strftime('%p').lower()
            )
        })
    for day in WEEKDAYS:
        if day[1] not in [open_day['name'] for open_day in days]:
            days.append({
                'name': str(day[1]),
                'hours': 'Closed'
            })

    if concise:
        # [{'hours': '9:00am to 5:00pm', 'day_names': u'Monday to Friday'}, {'hours':...
        template_name = 'openinghours/companyOpeningHoursListConcise.html'
        concise_days = []
        current_set = {}
        for day in days:
            if 'hours' not in current_set.keys():
                current_set = {'day_names': [day['name'],], 'hours': day['hours']}
            elif day['hours'] != current_set['hours']:
                concise_days.append(current_set)
                current_set = {'day_names': [day['name'],], 'hours': day['hours']}
            else:
                current_set['day_names'].append(day['name'])
        concise_days.append(current_set)

        for day_set in concise_days:
            if len(day_set['day_names']) > 2:
                day_set['day_names'] = '%s to %s' % (day_set['day_names'][0], day_set['day_names'][-1])
            elif len(day_set['day_names']) > 1:
                day_set['day_names'] = '%s and %s' % (day_set['day_names'][0], day_set['day_names'][-1])
            else:
                day_set['day_names'] = '%s' % day_set['day_names'][0]
        
        days = concise_days

    t = template.loader.get_template(template_name)
    return t.render(template.Context({
        'days': days
    }))
